# encoding: utf-8

# -------------------------------------------------------------------------------
# (C) Copyright 2019 - 2022 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors (“Micro Focus”) are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be
# liable for technical or editorial errors or omissions contained herein. The
# information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential
# information and a valid license is required for possession, use or copying. If
# this work is provided to the U.S. Government, consistent with FAR 12.211 and
# 12.212, Commercial Computer Software, Computer Software Documentation, and
# Technical Data for Commercial Items  are licensed to the U.S. Government under
# vendor's standard commercial license.
# ------------------------------------------------------------------------------
# Copyright (c) 2006 Novell, Inc. All Rights Reserved.
#
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of version 2 of the GNU General Public License as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, contact Novell, Inc.
#
# To contact Novell about this file by physical or electronic mail, you may find
# current contact information at www.novell.com.
# ------------------------------------------------------------------------------

# File:	modules/NovellCifs.ycp
# Package:	Configuration of novell-cifs
# Summary:	NovellCifs settings, input and output functions
# Authors:	Mark Clark <mclark@novell.com>
#
# $Id: NovellCifs.ycp 27914 2006-02-13 14:32:08Z locilka $
#
# Representation of the configuration of novell-cifs.
# Input and output routines.
require "yast"

module Yast
  class NovellCifsClass < Module
    def main
      textdomain "novell-cifs"

      Yast.import "Popup"
      Yast.import "Service"
      Yast.import "Progress"
      Yast.import "Report"
      Yast.import "Summary"
      Yast.import "Misc"
      Yast.import "NetworkInterfaces"
      Yast.import "Edirectory"
      Yast.import "Message"
      Yast.import "Mode"
      Yast.import "NovellUtils"
      Yast.import "NovellLDAP"
      Yast.import "NovellPatterns"
      Yast.import "NovellAddonInstall"
      Yast.import "Stage"
      Yast.import "String"
      Yast.import "Wizard"
      @auto_clone = false

      @edir_service_configured = false

      @cifs_bin_dir = "/opt/novell/cifs/bin"

      @schemaLogPath = "/var/opt/novell/eDirectory/log/oes_schema.log"

      @xad_packages_installed = false

      @xadMode = false

      @xadTestMode = false

      # Are we in upgrade mode?
      @upgradeMode = false

      @upgrade_from = ""

      # Are we in reinstall mode?
      @reinstallMode = false

      # Settings Map
      @Settings = {}
      @oldSettings = {}

      # Data was modified?
      @modified = false


      @proposal_valid = false

      # Write only, used during autoinstallation.
      # Don't run services and SuSEconfig, it's all done at one place.
      @write_only = false
      @globals_configured = false

      # Abort function
      # return boolean return true if abort
      @AbortFunction = fun_ref(method(:Modified), "boolean ()")

      @scr_conf_path = path(".sysconfig.novell-cifs")
    end

    # Abort function
    # @return [Boolean] return true if abort
    def Abort
      return @AbortFunction.call == true if @AbortFunction != nil
      false
    end

    # Get eDir context data from Cifs conf file
    def GetContextData
      data_file = "/etc/opt/novell/cifs/cifsctxs.conf"
      stmp = []
      sRet = []

      # First see if the file exists.
      if SCR.Read(path(".target.size"), data_file) == -1
        Builtins.y2milestone(
          "NovellCifs.GetContextData:cifsctxs conf file does not exist."
        )
        return nil # nil indicates empty or no file
      end

      # Read the file into memory
      schemaData = Convert.to_string(
        SCR.Read(path(".target.string"), data_file)
      )
      if schemaData != ""
        # remove extra LF characters on the end
        schemaData = NovellUtils.StripTrailing(schemaData, "\n")

        # remove extra LF characters on the front
        schemaData = NovellUtils.StripLeading(schemaData, "\n")
        # break up into individual lines
        stmp = Builtins.splitstring(schemaData, "\n")
        check = ""
        Builtins.foreach(stmp) do |item|
          item = NovellUtils.StripLeading(item, " ")
          check = Builtins.substring(item, 0, 1)
          next if item == nil || item == "" || check == "#"
          sRet = Builtins.add(sRet, item)
        end
      else
        sRet = nil # empty or no file;
        Builtins.y2milestone(
          "NovellCifs.GetContextData:cifsctxs conf file was empty."
        )
      end
      Builtins.y2milestone("NovellCifs.GetContextData:returning: %1", stmp)
      deep_copy(sRet)
    end

    # Retrievs the password only
    def RetrievCredentials
      creds = ""

      cmd = "/opt/novell/cifs/bin/retrieve_proxy_cred"

      Builtins.y2milestone("novell-cifs.RetrieveCredentials:executing: %1", cmd)

      val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      exit = Ops.get_integer(val, "exit", -1)
      Builtins.y2milestone(
        "novell-cifs.RetrieveCredentials:return code:%1",
        exit
      )
      if exit == 0
        creds = Ops.get_string(val, "stdout", "")
        # remove the line feed from the stdout string
        creds = Builtins.substring(
          creds,
          0,
          Ops.subtract(Builtins.size(creds), 1)
        )
      else
        creds = nil
      end

      creds
    end

    def getAdminData
      admin_fdn = NovellUtils.Nds2Ldap(
        Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
      )
      pwd = Ops.get_string(
        NovellLDAP.Settings,
        "CONFIG_LDAP_ADMIN_PASSWORD",
        ""
      )

      if @xadMode
        xad_tree_admin_context = Ops.get_string(
          NovellLDAP.Settings,
          "XAD_TREE_ADMIN_CONTEXT",
          ""
        )
        Builtins.y2milestone(
          "NovellCifs:XAD_TREE_ADMIN_CONTEXT:%1",
          xad_tree_admin_context
        )
        if xad_tree_admin_context != nil && xad_tree_admin_context != ""
          admin_fdn = NovellUtils.Nds2Ldap(xad_tree_admin_context)
          pwd = Ops.get_string(
            NovellLDAP.Settings,
            "XAD_TREE_ADMIN_PASSWORD",
            ""
          )
        end
      end

      adminData = []
      adminData = Builtins.add(adminData, admin_fdn)
      adminData = Builtins.add(adminData, pwd)

      Builtins.y2milestone("NovellCifs:admin:%1", Ops.get(adminData, 0, ""))

      deep_copy(adminData)
    end

    def VerifyUser(proxy_fdn, proxy_pwd)
      # This code is here for development purposes
      if Ops.greater_than(
          SCR.Read(path(".target.size"), "/opt/novell/oes-install/rundebug"),
          0
        )
        return 0
      end

      # Call GetSSCert in eDir so verify_user.sh will work
      Builtins.y2milestone("NovellCifs.VerifyUser:calling SSRootCert()")
      Edirectory.SSRootCert

      adminData = getAdminData
      admin = NovellUtils.Nds2Ldap(Ops.get(adminData, 0, ""))
      password = Ops.get(adminData, 1, "")

      proxy_fdn = NovellUtils.Nds2Ldap(proxy_fdn)

      ldapServer = ""
      ldapPort = -1

      server_list = NovellLDAP.GetRemoteServers(nil)
      if Ops.greater_than(Builtins.size(server_list), 0)
        ldapServer = Ops.get_string(
          Ops.get(server_list, 0, {}),
          "CONFIG_LDAP_SERVER_IP",
          ""
        )
        ldapPort = Ops.get_integer(
          Ops.get(server_list, 0, {}),
          "CONFIG_LDAP_SECURE_PORT",
          -1
        )
      else
        server_list = NovellLDAP.GetLocalServers(nil)
        ldapServer = Ops.get_string(
          Ops.get(server_list, 0, {}),
          "CONFIG_LDAP_SERVER_IP",
          ""
        )
        ldapPort = Ops.get_integer(
          Ops.get(server_list, 0, {}),
          "CONFIG_LDAP_SECURE_PORT",
          -1
        )
      end

      useExisting = "1"
      if Ops.get_string(@Settings, "CONFIG_CIFS_CREATE_NEW_PROXY_USER", "") == "yes"
        useExisting = "0"
      end

      useSsl = "0"
      if Ops.get_string(@Settings, "CONFIG_CIFS_USE_SECURE_PORT", "") == "yes"
        useSsl = "1"
      end

      # Usage: verify-user.sh <ldap server> <ldap port> <admin_fdn> <proxy_fdn> <ssl (0/1)> <user existing (0/1)>
      cmd = Builtins.sformat(
        "/opt/novell/cifs/bin/verify-user.sh %1 %2 '%3' '%4' %5 %6",
        ldapServer,
        ldapPort,
        admin,
        proxy_fdn,
        useSsl,
        useExisting
      )


      if password == nil || password == ""
        Builtins.y2milestone("novellcifs.VeridyUser:password was empty.")
      end

      if @xadTestMode
        Builtins.y2milestone(
          "novellcifs.VerifyUser:executing:%1",
          Builtins.sformat(
            "OES_DATA='%1' OES_DATA2='%2' %3",
            password,
            proxy_pwd,
            cmd
          )
        )
      else
        Builtins.y2milestone("novellcifs.VerifyUser:executing:%1", cmd)
      end

      ret = Convert.to_map(
        SCR.Execute(
          path(".target.bash_output"),
          Builtins.sformat(
            "OES_DATA='%1' OES_DATA2='%2' %3",
            password,
            proxy_pwd,
            cmd
          )
        )
      )
      Builtins.y2milestone("novellcifs.VerifyUser:Returned: %1", ret)
      exit = Ops.get_integer(ret, "exit", -1)

      exit
    end

    def AttachUserPolicy(policy_name)
      retVal = true

      adminData = getAdminData
      admin = NovellUtils.Nds2Ldap(Ops.get(adminData, 0, ""))
      password = Ops.get(adminData, 1, "")

      proxy_fdn = NovellUtils.Nds2Ldap(
        Ops.get_string(@Settings, "CONFIG_CIFS_PROXY_USER", "")
      )

      # Usage :  attach_user_pwpolicy.sh <user_fdn> <pw_policy_name> <base_fdn> <basefdn_pwd>
      cmd = Ops.add(
        Ops.add(
          Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(@cifs_bin_dir, "/attach_user_pwpolicy.sh '"),
                    proxy_fdn
                  ),
                  "' "
                ),
                policy_name
              ),
              "' "
            ),
            admin
          ),
          "' "
        ),
        password
      )
      log_cmd = Ops.add(
        Ops.add(
          Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(@cifs_bin_dir, "/attach_user_pwpolicy.sh '"),
                    proxy_fdn
                  ),
                  "' "
                ),
                policy_name
              ),
              "' "
            ),
            admin
          ),
          "' "
        ),
        "****"
      )

      ret = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))

      if password == nil || password == ""
        Builtins.y2milestone(
          "novellcifs.AttachUserAsPolicy:password was empty."
        )
      end

      if @xadTestMode
        Builtins.y2milestone(
          "novellcifs.AttachUserPolicy:executing:%1, Returned: %2",
          cmd,
          ret
        )
      else
        Builtins.y2milestone(
          "novellcifs.AttachUserPolicy:executing:%1, Returned: %2",
          log_cmd,
          ret
        )
      end

      exit = Ops.get_integer(ret, "exit", -1)
      if exit != 0
        retVal = false
        Popup.Message(
          Builtins.sformat(
            _("Unable to attach user password policy %1, Error: %2"),
            policy_name,
            exit
          )
        )
      end
      retVal
    end

    # Data was modified?
    # @return true if modified
    def Modified
      Builtins.y2debug("modified=%1", @modified)
      @modified
    end
    def ReadOption(optname, replace)
      optvalue = Convert.to_string(
        SCR.Read(Ops.add(@scr_conf_path, Builtins.topath(optname)))
      )

      optvalue = "" if optvalue == nil
      optvalue = replace if optvalue == "" && replace != nil

      optvalue
    end

    # Read all novell-cifs settings
    # @return true on success
    def Read
      # NovellCifs read dialog caption
      caption = _("Initializing novell-cifs Configuration")

      steps = 4

      sl = 500
      Builtins.sleep(sl)

      # We do not set help text here, because it was set outside
      Progress.New(
        caption,
        " ",
        steps,
        [
          # Progress stage 1/1
          _("Read OES CIFS Services settings"),
          _("Checking dependencies")
        ],
        [
          # Progress step 1/1
          _("Reading OES CIFS Services settings..."),
          _("Checking dependencies"),
          # Progress finished
          _("Finished")
        ],
        ""
      )

      # read database
      return false if Abort()
      Progress.NextStage

      # Error message
      Report.Error(_("Cannot read database1.")) if false
      Builtins.sleep(sl)

      # First, get the default LDAP/eDir data
      progress_orig = Progress.set(false)
      NovellLDAP.Read
      Progress.set(progress_orig)

      # See if XAD is installed. If so, default the data.
      @xad_packages_installed = NovellPatterns.IsPatternInstalled(
        "novell-domain-services"
      )

      @xad_packages_installed = true if @xadTestMode

      Builtins.y2milestone(
        "NovellCifs:xad_packages_installed:%1",
        @xad_packages_installed
      )
      if @xad_packages_installed
        if Ops.get_string(Edirectory.EdirSettings, "XAD_SERVICE_CONFIGURED", "") == "yes" ||
            Convert.to_string(
              SCR.Read(path(".sysconfig.novell.xad.SERVICE_CONFIGURED"))
            ) == "yes"
          @xadMode = true
        end
        Builtins.y2milestone(
          "NovellCifs:xad configured from eDir map: %1",
          Ops.get_string(Edirectory.EdirSettings, "XAD_SERVICE_CONFIGURED", "")
        )
        Builtins.y2milestone(
          "NovellCifs:xad configured from file: %1",
          Convert.to_string(
            SCR.Read(path(".sysconfig.novell.xad.SERVICE_CONFIGURED"))
          )
        )
      end

      Builtins.y2milestone("NovellCifs:xadMode:%1", @xadMode)

      @upgrade_from = NovellUtils.UpgradeIsFrom
      if (Mode.update || Mode.autoupgrade) &&  @upgrade_from != ""
        @oldSettings = NovellUtils.GetOldSysconfigMap("novell-cifs")
        @upgradeMode = true
      end

      Builtins.y2milestone(
        "NovelLCifs:
        upgradeFrom = #{@upgrade_from}"
      )
      Builtins.y2milestone("NovelLCifs:oldSettings: %1", @oldSettings)

      # determine if Novell CIFS Services configuration already exists
      service_configured = Builtins.tolower(
        ReadOption("SERVICE_CONFIGURED", "no")
      )

      if @upgradeMode
        Ops.set(
          @Settings,
          "CONFIG_CIFS_LDAP_SERVER",
          Ops.get_string(@oldSettings, "CONFIG_CIFS_LDAP_SERVER", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_CIFS_SERVER_CONTEXT",
          Ops.get_string(@oldSettings, "CONFIG_CIFS_SERVER_CONTEXT", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_CIFS_USE_SECURE_PORT",
          Ops.get_string(@oldSettings, "CONFIG_CIFS_USE_SECURE_PORT", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_CIFS_USE_OCS_FOR_CREDS",
          Ops.get_string(@oldSettings, "CONFIG_CIFS_USE_OCS_FOR_CREDS", "")
        )

        intval = Builtins.tointeger(
          Ops.get_string(@oldSettings, "CONFIG_CIFS_LDAP_PORT", "")
        )
        intval = 636 if intval == nil || Ops.less_or_equal(intval, 0) # make sure val is not nil
        Ops.set(@Settings, "CONFIG_CIFS_LDAP_PORT", intval)

        Ops.set(
          @Settings,
          "CONFIG_CIFS_PROXY_USER",
          Ops.get_string(@oldSettings, "CONFIG_CIFS_PROXY_USER", "")
        )

        creds = RetrievCredentials()

        creds = "" if creds == nil
        Ops.set(@Settings, "CONFIG_CIFS_PROXY_USER_PASSWORD", creds)

        Ops.set(@Settings, "CONFIG_CIFS_EDIR_CONTEXTS", []) # we don't get this from the sysconfig file

        # for this product we do not use the /etc/sysconfig/novell/NvlCifs file to read the eDir contexts. We use the cifs config file.
        conf_context_list = GetContextData() # Get context data from the CIFS conf file if it exists.
        if conf_context_list != nil
          Builtins.foreach(conf_context_list) do |item|
            item = NovellUtils.Ldap2Nds(item)
            if !Builtins.contains(
                Ops.get_list(@Settings, "CONFIG_CIFS_EDIR_CONTEXTS", []),
                item
              )
              Ops.set(
                @Settings,
                "CONFIG_CIFS_EDIR_CONTEXTS",
                Builtins.add(
                  Ops.get_list(@Settings, "CONFIG_CIFS_EDIR_CONTEXTS", []),
                  item
                )
              )
            end
          end
        else
          Ops.set(@Settings, "CONFIG_CIFS_EDIR_CONTEXTS", [])
        end

        Ops.set(
          @Settings,
          "CONFIG_CIFS_USERCONTEXT_RIGHTS",
          Ops.get_string(@oldSettings, "CONFIG_CIFS_USERCONTEXT_RIGHTS", "")
        )
        if Ops.get_string(@Settings, "CONFIG_CIFS_USERCONTEXT_RIGHTS", "yes") == "" ||
            Ops.get_string(@Settings, "CONFIG_CIFS_USERCONTEXT_RIGHTS", "yes") == nil
          Ops.set(@Settings, "CONFIG_CIFS_USERCONTEXT_RIGHTS", "yes")
        end


        #       Settings["CONFIG_CIFS_SUBTREE_SEARCH"] = oldSettings["CONFIG_CIFS_SUBTREE_SEARCH"]:"";
        #	   if(Settings["CONFIG_CIFS_SUBTREE_SEARCH"]:"no" == "" || Settings["CONFIG_CIFS_SUBTREE_SEARCH"]:"no" == nil)
        #	   Settings["CONFIG_CIFS_SUBTREE_SEARCH"] = "no";

        cmd = "grep \"SUBTREE\" \\/etc\\/opt\\/novell\\/cifs\\/cifs.conf | expand |tr -s \" \"|cut -d \" \" -f2"
        retval = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
        Ops.set(
          @Settings,
          "CONFIG_CIFS_SUBTREE_SEARCH",
          NovellUtils.CleanUpString(Ops.get_string(retval, "stdout", ""))
        )
        if Ops.get_string(@Settings, "CONFIG_CIFS_SUBTREE_SEARCH", "no") == "" ||
            Ops.get_string(@Settings, "CONFIG_CIFS_SUBTREE_SEARCH", "no") == nil
          Ops.set(@Settings, "CONFIG_CIFS_SUBTREE_SEARCH", "no")
        end

        Builtins.y2milestone("cmd executd is %1", cmd)
        Builtins.y2milestone("map retval is %1", retval)


        Ops.set(@Settings, "CONFIG_CIFS_CREATE_NEW_PROXY_USER", "no") # proxy user will already exist on an upgrade. # standard install
      else
        ldapPort = 636
        # Next get Novell CIFS Services data
        servername = ReadOption("CONFIG_CIFS_LDAP_SERVER", "")
        if servername == ""
          server_list = NovellLDAP.GetLocalServers(nil)
          if Ops.greater_than(Builtins.size(server_list), 0)
            servername = Ops.get_string(
              Ops.get(server_list, 0, {}),
              "CONFIG_LDAP_SERVER_IP",
              ""
            )
            ldapPort = Ops.get_integer(
              Ops.get(server_list, 0, {}),
              "CONFIG_LDAP_SECURE_PORT",
              636
            )
          else
            server_list = NovellLDAP.GetRemoteServers(nil)
            servername = Ops.get_string(
              Ops.get(server_list, 0, {}),
              "CONFIG_LDAP_SERVER_IP",
              ""
            )
            ldapPort = Ops.get_integer(
              Ops.get(server_list, 0, {}),
              "CONFIG_LDAP_SECURE_PORT",
              636
            )
          end
        end
        Ops.set(
          @Settings,
          "CONFIG_CIFS_LDAP_PORT",
          Builtins.tointeger(
            ReadOption(
              "CONFIG_CIFS_LDAP_PORT",
              Builtins.sformat("%1", ldapPort)
            )
          )
        )

        adminData = getAdminData
        admin = NovellUtils.Nds2Ldap(Ops.get(adminData, 0, ""))

        ldapadmincontext = admin

        info = NovellAddonInstall.GetProductInfo("edirectory")
        if Ops.get_boolean(info, "SERVICE_CONFIGURED", false) == true
          @edir_service_configured = true
        else
          @edir_service_configured = false
        end

        edirservercontext = ""
        if Mode.installation && Stage.normal && @edir_service_configured == true
          edirservercontext = Misc.SysconfigRead(
            path(".sysconfig.edirectory.CONFIG_EDIR_SERVER_CONTEXT"),
            ""
          )
          Builtins.y2milestone(
            "cifs:edir server context from file (post install): %1",
            edirservercontext
          )
        else
          edirservercontext = Ops.get_string(
            Edirectory.EdirSettings,
            "CONFIG_EDIR_SERVER_CONTEXT",
            ""
          )
          Builtins.y2milestone(
            "cifs:edir server context from map): %1",
            edirservercontext
          )
          if edirservercontext == nil || edirservercontext == ""
            edirservercontext = Misc.SysconfigRead(
              path(".sysconfig.edirectory.CONFIG_EDIR_SERVER_CONTEXT"),
              ""
            )
            Builtins.y2milestone(
              "cifs:edir server context from file: %1",
              edirservercontext
            )
          end
        end

        if edirservercontext == nil || edirservercontext == ""
          edirservercontext = NovellUtils.GetContextNDS(ldapadmincontext)
        end

        Ops.set(@Settings, "CONFIG_CIFS_LDAP_SERVER", servername)
        Ops.set(
          @Settings,
          "CONFIG_CIFS_SERVER_CONTEXT",
          ReadOption("CONFIG_CIFS_SERVER_CONTEXT", edirservercontext)
        )
        Ops.set(
          @Settings,
          "CONFIG_CIFS_USE_SECURE_PORT",
          Builtins.tolower(ReadOption("CONFIG_CIFS_USE_SECURE_PORT", "yes"))
        )
        Ops.set(
          @Settings,
          "CONFIG_CIFS_USE_OCS_FOR_CREDS",
          Builtins.tolower(ReadOption("CONFIG_CIFS_USE_OCS_FOR_CREDS", "yes"))
        )

        Ops.set(
          @Settings,
          "CONFIG_CIFS_CREATE_NEW_PROXY_USER",
          Builtins.tolower(
            ReadOption("CONFIG_CIFS_CREATE_NEW_PROXY_USER", "yes")
          )
        )
        if service_configured == "yes"
          Ops.set(@Settings, "CONFIG_CIFS_CREATE_NEW_PROXY_USER", "no")
        end # The user was created or used in the initial install.

        useCommonProxy = Ops.get_string(
          NovellLDAP.Settings,
          "CONFIG_LDAP_DO_PROXY",
          ""
        )
        Builtins.y2milestone(
          "NovellCifs.Read:useCommonProxy: %1",
          useCommonProxy
        )

        commonProxyName = ""
        if useCommonProxy == "yes"
          commonProxyName = NovellUtils.Nds2Ldap(
            Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", "")
          )
          Builtins.y2milestone(
            "NovellCifs.Read:commonProxyName: '%1'",
            commonProxyName
          )
          commonProxyPassword = Ops.get_string(
            NovellLDAP.Settings,
            "CONFIG_LDAP_PROXY_PASSWORD",
            ""
          )
        end

        if service_configured != "yes" # new install mode
          hostName = NovellUtils.GetHostname
          hostName = "" if hostName == nil

          Ops.set(
            @Settings,
            "CONFIG_CIFS_PROXY_USER",
            Ops.add(
              Ops.add(Ops.add("cn=cifsProxyUser-", hostName), ","),
              NovellUtils.Nds2Ldap(edirservercontext)
            )
          )
          Ops.set(
            @Settings,
            "CONFIG_CIFS_PROXY_USER_DEFAULT",
            Ops.add(
              Ops.add(Ops.add("cn=cifsProxyUser-", hostName), ","),
              NovellUtils.Nds2Ldap(edirservercontext)
            )
          )
          pWord = NovellUtils.GenerateRandomPassword # generate a random password between 20 and 50 characters.
          pWord = Builtins.substring(pWord, 0, 12) # limit it to 12 characters so default password policy will work.
          Ops.set(@Settings, "CONFIG_CIFS_PROXY_USER_PASSWORD", pWord)
          Ops.set(@Settings, "CONFIG_CIFS_PROXY_USER_DEFAULT_PASSWORD", pWord)

          if useCommonProxy == "yes"
            Ops.set(@Settings, "CONFIG_CIFS_PROXY_USER", commonProxyName)
            if commonProxyPassword != ""
              Ops.set(
                @Settings,
                "CONFIG_CIFS_PROXY_USER_PASSWORD",
                commonProxyPassword
              )
            else
              Ops.set(
                @Settings,
                "CONFIG_CIFS_PROXY_USER_PASSWORD",
                NovellUtils.RetrieveProxyCredentials("password")
              )
            end
            Ops.set(@Settings, "CONFIG_CIFS_COMMON_PROXY", commonProxyName)
            Ops.set(
              @Settings,
              "CONFIG_CIFS_COMMON_PROXY_PASSWORD",
              Ops.get_string(@Settings, "CONFIG_CIFS_PROXY_USER_PASSWORD", "")
            )
            Ops.set(@Settings, "CONFIG_CIFS_CREATE_NEW_PROXY_USER", "no")
          end

          Ops.set(
            @Settings,
            "CONFIG_CIFS_EDIR_CONTEXTS",
            Builtins.splitstring(edirservercontext, "#")
          ) # the config file exists, so use the data. This is reconfigure mode.
        else
          Ops.set(
            @Settings,
            "CONFIG_CIFS_PROXY_USER",
            ReadOption("CONFIG_CIFS_PROXY_USER", "")
          )
          if useCommonProxy == "yes"
            if commonProxyPassword == ""
              commonProxyPassword = NovellUtils.RetrieveProxyCredentials(
                "password"
              )
            end

            if NovellUtils.Nds2Ldap(
                Ops.get_string(@Settings, "CONFIG_CIFS_PROXY_USER", "")
              ) ==
                NovellUtils.Nds2Ldap(commonProxyName)
              Ops.set(
                @Settings,
                "CONFIG_CIFS_PROXY_USER_PASSWORD",
                commonProxyPassword
              )
            end
            Ops.set(@Settings, "CONFIG_CIFS_COMMON_PROXY", commonProxyName)
            Ops.set(
              @Settings,
              "CONFIG_CIFS_COMMON_PROXY_PASSWORD",
              commonProxyPassword
            )
            Ops.set(@Settings, "CONFIG_CIFS_CREATE_NEW_PROXY_USER", "no")
          else
            creds = RetrievCredentials()
            creds = "" if creds == nil

            Ops.set(@Settings, "CONFIG_CIFS_PROXY_USER_PASSWORD", creds)
          end

          # for this product we do not use the /etc/sysconfig/novell/NvlCifs file to read the eDir contexts. We use the cifs config file.
          conf_context_list = GetContextData() # Get context data from the CIFS conf file if it exists.
          if conf_context_list != nil
            Builtins.foreach(conf_context_list) do |item|
              item = NovellUtils.Ldap2Nds(item)
              if !Builtins.contains(
                  Ops.get_list(@Settings, "CONFIG_CIFS_EDIR_CONTEXTS", []),
                  item
                )
                Ops.set(
                  @Settings,
                  "CONFIG_CIFS_EDIR_CONTEXTS",
                  Builtins.add(
                    Ops.get_list(@Settings, "CONFIG_CIFS_EDIR_CONTEXTS", []),
                    item
                  )
                )
              end
            end
          else
            Ops.set(@Settings, "CONFIG_CIFS_EDIR_CONTEXTS", [])
          end
          Ops.set(
            @Settings,
            "CONFIG_CIFS_USERCONTEXT_RIGHTS",
            ReadOption("CONFIG_CIFS_USERCONTEXT_RIGHTS", "yes")
          )
          Ops.set(
            @Settings,
            "CONFIG_CIFS_SUBTREE_SEARCH",
            ReadOption("CONFIG_CIFS_SUBTREE_SEARCH", "no")
          )
        end
      end
      @globals_configured = true
      Builtins.y2debug("novell-cifs READ Complete: %1", @Settings)

      # Error message
      if false
        Report.Error(_("Cannot read current Novell DHCP Services settings."))
      end
      Builtins.sleep(sl)

      # check dependencies
      return false if Abort()
      Progress.NextStage

      return false if Abort()
      Progress.NextStage
      Progress.Title(_("Finished"))
      Builtins.sleep(sl)

      return false if Abort()
      @modified = false
      true
    end

    # insufficient_rights_error()
    #
    # Checks a string for output indicating insufficient rights.
    def insufficient_rights_error(msg)
      return false if msg == nil || msg == ""

      idx = Builtins.search(msg, "Insufficient access")
      return true if Ops.greater_or_equal(idx, 0)

      idx = Builtins.search(msg, "no access (-672)")
      return true if Ops.greater_or_equal(idx, 0)

      false
    end

    # Open Cifs Port
    def OpenCifsPort
      Builtins.y2milestone("NovellCifs.OpenCifsPort: Opening Cifs Port.")
      NovellUtils.SetFirewallPort(137, :udp, true)
      NovellUtils.SetFirewallPort(138, :udp, true)
      NovellUtils.SetFirewallPort(139, :tcp, true)
      NovellUtils.SetFirewallPort(445, :tcp, true)
    end

    # Write all novell-cifs settings
    # @return true on success
    def Write
      Builtins.y2milestone("NovellCifs.Write:entering write.")
      Wizard.DisableAbortButton

      pattern = "novell-cifs"
      if NovellPatterns.IsPatternInstalled(pattern) == false
        Builtins.y2milestone(
          "The required pattern, %1, is not installed.",
          pattern
        )
        if Mode.normal
          Report.Error(
            Builtins.sformat(
              _(
                "Configuration aborted. Install the required %1 pattern and try again."
              ),
              pattern
            )
          )
        end

        return true
      end
      # Novell CIFS Services write dialog caption
      if !Ops.get(NovellAddonInstall.products_to_configure, "novell-cifs", true)
        Builtins.y2milestone("Novell CIFS -- configure later")
        return true
      end
      abortInstall = Builtins.tolower(
        NovellUtils.ReadOption("ABORT_OES_INSTALL")
      )
      if abortInstall == "yes"
        Builtins.y2milestone(
          "NovellLum:setting ABORT_OES_INSTALL is set to 'yes'."
        )
        return true
      end


      caption = _("Saving OES CIFS Services Configuration")

      steps = 1

      sl = 500
      Builtins.sleep(sl)

      # We do not set help text here, because it was set outside
      Progress.New(
        caption,
        " ",
        steps,
        [
          # Progress stage 1/1
          _("Write OES CIFS Services settings")
        ],
        [
          # Progress step 1/1
          _("Writing OES CIFS Services settings..."),
          # Progress finished
          _("Finished")
        ],
        ""
      )

      # write settings
      return false if Abort()
      Progress.NextStage

      # SCR::Write(.sysconfig.novell-cifs.SERVICE_CONFIGURED, "no");

      cmd = ""
      rval = 0
      adminData = getAdminData
      admin = NovellUtils.Nds2Ldap(Ops.get(adminData, 0, ""))
      ldapAdmin = NovellUtils.Nds2Ldap(admin)
      password = Ops.get(adminData, 1, "")
      full_admin_context = NovellUtils.Ldap2Nds(admin)
      admin_pwd = password
      ssl = "0"
      if Ops.get_string(@Settings, "CONFIG_CIFS_USE_SECURE_PORT", "") != "no"
        ssl = "1"
      end

      if Mode.autoinst
        commonProxyName = NovellUtils.Nds2Ldap(
          Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", "")
        )
        if commonProxyName != ""
          Ops.set(@Settings, "CONFIG_CIFS_PROXY_USER", commonProxyName)
        else
          Ops.set(@Settings, "CONFIG_CIFS_PROXY_USER", NovellUtils.RetrieveProxyCredentials("username"))
        end
        commonProxyPassword = Ops.get_string(NovellLDAP.Settings,"CONFIG_LDAP_PROXY_PASSWORD","")
        if commonProxyPassword != ""
          Ops.set(@Settings, "CONFIG_CIFS_PROXY_USER_PASSWORD", commonProxyPassword)
        else
          Ops.set(@Settings, "CONFIG_CIFS_PROXY_USER_PASSWORD", NovellUtils.RetrieveProxyCredentials("password"))
        end
      end

      # Nothing but an RPM update to be done
      if !@upgradeMode
        upgradeStr = ""
        upgradeStr = "UPGRADE='1' " if @upgradeMode

        # determine IP addresses
        server_list = Ops.get_list(
          NovellLDAP.Settings,
          "CONFIG_LDAP_SERVERS_LIST",
          []
        )
        primary_server = NovellLDAP.GetAddressServer(
          Ops.get_string(@Settings, "CONFIG_CIFS_LDAP_SERVER", ""),
          server_list
        )
        fallback_list = NovellLDAP.GetFallbackServerList(
          Ops.get_string(@Settings, "CONFIG_CIFS_LDAP_SERVER", ""),
          server_list
        )
        Builtins.y2milestone("Primary server: %1", primary_server)
        Builtins.y2milestone("Fallback server_list: %1", fallback_list)


        # Extend CIFS Services schema.
        insufficient_rights = false
        schema_file = "/opt/novell/cifs/schema/nfap.sch"
        ndssch_file = "/opt/novell/eDirectory/bin/ndssch"
        cmd = Ops.add(
          Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        Ops.add(ndssch_file, " -h "),
                        Ops.get_string(
                          primary_server,
                          "CONFIG_LDAP_SERVER_IP",
                          ""
                        )
                      ),
                      " -F '"
                    ),
                    @schemaLogPath
                  ),
                  "' '"
                ),
                full_admin_context
              ),
              "'"
            ),
            " "
          ),
          schema_file
        )

        if admin_pwd == nil || admin_pwd == ""
          Builtins.y2milestone("novellcifs.write:password was empty.")
        end

        if @xadTestMode
          Builtins.y2milestone(
            "novell-cifs:Executing %1",
            Ops.add(Ops.add(Ops.add("ADM_PASSWD='", admin_pwd), "' "), cmd)
          )
        else
          Builtins.y2milestone("novell-cifs:Executing %1", cmd)
        end

        rmap = Convert.to_map(
          SCR.Execute(
            path(".target.bash_output"),
            Ops.add(Ops.add(Ops.add("ADM_PASSWD='", admin_pwd), "' "), cmd)
          )
        )
        if Ops.get_integer(rmap, "exit", -1) != 0
          if insufficient_rights_error(Ops.get_string(rmap, "stderr", "")) == true
            insufficient_rights = true
          end

          Builtins.y2milestone(
            "novell-cifs:Failed (error=%1) to extend schema using default IP address / hostname: %2, using file %3",
            Ops.get_integer(rmap, "exit", -1),
            Ops.get_string(primary_server, "CONFIG_LDAP_SERVER_IP", ""),
            schema_file
          )

          Builtins.foreach(fallback_list) do |server_info|
            cmd = Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        Ops.add(
                          Ops.add(
                            Ops.add(ndssch_file, " -h "),
                            Ops.get_string(
                              server_info,
                              "CONFIG_LDAP_SERVER_IP",
                              ""
                            )
                          ),
                          " -F '"
                        ),
                        @schemaLogPath
                      ),
                      "' '"
                    ),
                    full_admin_context
                  ),
                  "'"
                ),
                " "
              ),
              schema_file
            )
            if admin_pwd == nil || admin_pwd == ""
              Builtins.y2milestone("novellcifs.write2:password was empty.")
            end
            if @xadTestMode
              Builtins.y2milestone(
                "novell-cifs:Retry of schema extension with fallback IP address / hostname:Executing %1",
                Ops.add(Ops.add(Ops.add("ADM_PASSWD='", admin_pwd), "' "), cmd)
              )
            else
              Builtins.y2milestone(
                "novell-cifs:Retry of schema extension with fallback IP address / hostname:Executing %1",
                cmd
              )
            end
            rmap = Convert.to_map(
              SCR.Execute(
                path(".target.bash_output"),
                Ops.add(Ops.add(Ops.add("ADM_PASSWD='", admin_pwd), "' "), cmd)
              )
            )
            if Ops.get_integer(rmap, "exit", -1) != 0
              if insufficient_rights_error(Ops.get_string(rmap, "stderr", "")) == true
                insufficient_rights = true
              end
              Builtins.y2milestone(
                "novell-cifs:Failed (error=%1) to extend schema using fallback IP address / hostname: %2, using file %3",
                Ops.get_integer(rmap, "exit", -1),
                Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", ""),
                schema_file
              )
            else
              raise Break
            end
          end
        end
        if Ops.get_integer(rmap, "exit", -1) != 0
          warn = Ops.add(
            Builtins.sformat(
              _("The user %1 has insufficient rights to extend schema.\n"),
              full_admin_context
            ),
            _(
              "This may not be a problem if joining a tree where the schema is already extended."
            )
          )
          Builtins.y2milestone("novell-cifs.write:%1", warn)
          Builtins.y2milestone(
            "novell-cifs.write:Failed to extend schema for Novell CIFS Services. Schema may already be extended if joining an existing Tree."
          )
        end

        # Call the Novell CIFS Services config module
        ldapServer = Ops.get_string(primary_server, "CONFIG_LDAP_SERVER_IP", "")
        ldapPort = Ops.get_integer(@Settings, "CONFIG_CIFS_LDAP_PORT", -1)

        proxyUser = NovellUtils.Nds2Ldap(
          Ops.get_string(@Settings, "CONFIG_CIFS_PROXY_USER", "")
        )
        proxyPassword = Ops.get_string(
          @Settings,
          "CONFIG_CIFS_PROXY_USER_PASSWORD",
          ""
        )

        useCommonProxy = Ops.get_string(
          NovellLDAP.Settings,
          "CONFIG_LDAP_DO_PROXY",
          ""
        )
        Builtins.y2milestone(
          "NovellCifs.Write:useCommonProxy: %1",
          useCommonProxy
        )
        if useCommonProxy == "yes"
          commonProxyName = NovellUtils.Nds2Ldap(
            Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", "")
          )
          commonProxyPassword = NovellUtils.RetrieveProxyCredentials("password")
          if commonProxyPassword == nil || commonProxyPassword == ""
            commonProxyPassword = Ops.get_string(
              NovellLDAP.Settings,
              "CONFIG_LDAP_PROXY_PASSWORD",
              ""
            )
          end

          if NovellUtils.Nds2Ldap(
              Ops.get_string(@Settings, "CONFIG_CIFS_PROXY_USER", "")
            ) == commonProxyName
            proxyPassword = commonProxyPassword
            Ops.set(@Settings, "CONFIG_CIFS_PROXY_USER_PASSWORD", proxyPassword)
          end
        end

        serverContext = NovellUtils.Nds2Ldap(
          Ops.get_string(@Settings, "CONFIG_CIFS_SERVER_CONTEXT", "")
        )
        ocs = "0"

        useExisting = "1"
        if Ops.get_string(@Settings, "CONFIG_CIFS_CREATE_NEW_PROXY_USER", "") == "yes"
          useExisting = "0"
        end

        if Ops.get_string(@Settings, "CONFIG_CIFS_USE_OCS_FOR_CREDS", "") != "no"
          ocs = "1"
        end
        usercontext_rights = Ops.get_string(
          @Settings,
          "CONFIG_CIFS_USERCONTEXT_RIGHTS",
          "yes"
        )
        usercontext_rights_value = 1
        usercontext_rights_value = 0 if usercontext_rights == "no"

        subtree_search = Ops.get_string(
          @Settings,
          "CONFIG_CIFS_SUBTREE_SEARCH",
          "no"
        )
        subtree_search_value = 0
        subtree_search_value = 1 if subtree_search == "yes"

        cmd = Builtins.sformat(
          "/opt/novell/cifs/bin/cifs-config.sh %1 %2 '%3' '%4' '%5' %6 %7 %8 %9",
          ldapServer,
          ldapPort,
          ldapAdmin,
          proxyUser,
          serverContext,
          ssl,
          ocs,
          subtree_search_value,
          usercontext_rights_value
        )


        Builtins.foreach(
          Ops.get_list(@Settings, "CONFIG_CIFS_EDIR_CONTEXTS", [])
        ) do |item|
          if !NovellValidators.ValidateContextExists(
            item,
            ldapServer,
            Builtins.sformat("%1", ldapPort),
            EdirUtils.GetLeafNameNDS(
              Ops.get_string(
                NovellLDAP.Settings,
                "CONFIG_LDAP_ADMIN_CONTEXT",
                ""
              )
            ),
            Ops.get_string(
              NovellLDAP.Settings,
              "CONFIG_LDAP_ADMIN_PASSWORD",
              ""
            )
          )
            Report.Error(
              Builtins.sformat(
                _("CIFS : Failed to assign the Proxy User as a trustee of %1"),
                item
              )
            )
            Builtins.y2milestone("novell-cifs.write: %1 Context does not exist", item)
          else
            cmd = Ops.add(
              Ops.add(Ops.add(cmd, " '"), NovellUtils.Ldap2Nds(item)),
              "'"
            )
          end
        end

        if admin_pwd == nil || admin_pwd == ""
          Builtins.y2milestone("novellcifs.write3:password was empty.")
        end

        if @xadTestMode
          Builtins.y2milestone(
            "novell-cifs.write:Executing %1",
            Builtins.sformat(
              "OES_DATA='%1' OES_DATA2='%2' %3",
              admin_pwd,
              proxyPassword,
              cmd
            )
          )
        else
          Builtins.y2milestone("novell-cifs.write:Executing %1", cmd)
        end

        retval = Convert.to_map(
          SCR.Execute(
            path(".target.bash_output"),
            Builtins.sformat(
              "%1OES_DATA='%2' OES_DATA2='%3' %4",
              upgradeStr,
              admin_pwd,
              proxyPassword,
              cmd
            )
          )
        )
        Builtins.y2milestone(
          "novellCifs::Execute cifs-config return code:\n%1",
          Ops.get_integer(retval, "exit", -1)
        )
        if Ops.get_integer(retval, "exit", -1) != 0
          ret2 = Ops.get_integer(retval, "exit", -1)
          errorMsg = ""
          errorMsg = _("Failed to create proxy user.") if ret2 == 1
          if ret2 == 2
            errorMsg = _("Failed to store proxy user credentials.")
          elsif ret2 == 3
            errorMsg = _(
              "Failed to assign eDirectory rights to the proxy user."
            )
          end

          Builtins.y2error(errorMsg)
          Report.Error(errorMsg)
        end
      end

      # Set firewall ports
      OpenCifsPort()

      # running nmasinst for CIFS NMAS method
      if Mode.update || Mode.autoupgrade
        nmaspath = "/opt/novell/cifs/share/nmasmthd/ntlm/config.txt"
        password2 = Ops.get_string(
          NovellLDAP.Settings,
          "CONFIG_LDAP_ADMIN_PASSWORD",
          ""
        )
        tree = Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_TREE_NAME", "")
        admin2 = NovellUtils.Ldap2Nds(
          Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
        )

        cmd = Builtins.sformat(
          "/opt/novell/eDirectory/bin/nmasinst -addmethod '%1' '%2' %3 -checkversion",
          admin2,
          tree,
          nmaspath
        )
        Builtins.y2milestone("NovellCifs:Executing %1", cmd)
        rmap = Convert.to_map(
          SCR.Execute(
            path(".target.bash_output"),
            Ops.add(Ops.add(Ops.add("OES_DATA='", password2), "' "), cmd)
          )
        )
        if !(Ops.get_integer(rmap, "exit", -1) == 0 ||
            Ops.get_integer(rmap, "exit", -1) == 93)
          Report.Error(
            Builtins.sformat(
              _(
                "Could not run nmasinst for CIFS NMAS method. Refer to\n/usr/share/YaST2/y2log file for more information"
              )
            )
          )
          Builtins.y2milestone("Output : %1", rmap)
        else
          Builtins.y2milestone("Output : %1", rmap)
        end
      end

      if Mode.update || Mode.autoupgrade

        cifs_proxyuser = NovellUtils.Nds2Ldap(Ops.get_string(@Settings, "CONFIG_CIFS_PROXY_USER", ""))
        proxyUser = NovellUtils.Nds2Ldap(
              Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", ""))

        if cifs_proxyuser == proxyUser
          Builtins.y2milestone("NovellCifs:Setting commonproxy username and password during upgrade")
          proxyPassword = NovellUtils.RetrieveProxyCredentials(
           "password"
          )

          ret = NovellUtils.StoreCredsInOCSforUpgrade(proxyUser, proxyPassword,"novell-cifs")
          if ret != 0
            Popup.Message(
            Builtins.sformat(_("Error storing credentials in OCS for Common Proxy User: %1"),
            ret ))
            Builtins.y2milestone(
            "Novell-cifs.ConfigureProxyUser:Error storing credentials in OCS for common Proxy User: %1",
            ret)
          end
        else
          Builtins.y2milestone("NovellCifs:CIFS using service proxy, moving to common proxy")
          NovellUtils.MovetoCommonProxyDuringUpgrade("novell-cifs")
          Ops.set(@Settings, "CONFIG_CIFS_PROXY_USER", Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", ""))
        end
      end
      if Mode.update || Mode.autoupgrade
        cifs_proxyuser = NovellUtils.Nds2Ldap(Ops.get_string(@Settings, "CONFIG_CIFS_PROXY_USER", ""))
        proxyUser = NovellUtils.Nds2Ldap(
                       Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", ""))
        if cifs_proxyuser != proxyUser
          Builtins.y2milestone("NovellCifs:CIFS using service proxy, moving to common proxy")
          NovellUtils.MovetoCommonProxyDuringUpgrade("novell-cifs")
          Ops.set(@Settings, "CONFIG_CIFS_PROXY_USER", Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", ""))
        end
      end

     # Write the /etc/sysconfig file
      SCR.Write(
        path(".sysconfig.novell-cifs.CONFIG_CIFS_LDAP_SERVER"),
        Builtins.sformat(
          "%1",
          Ops.get_string(@Settings, "CONFIG_CIFS_LDAP_SERVER", "")
        )
      )
      SCR.Write(
        path(".sysconfig.novell-cifs.CONFIG_CIFS_SERVER_CONTEXT"),
        NovellUtils.Nds2Ldap(
          Ops.get_string(@Settings, "CONFIG_CIFS_SERVER_CONTEXT", "")
        )
      )
      SCR.Write(
        path(".sysconfig.novell-cifs.CONFIG_CIFS_USE_SECURE_PORT"),
        Ops.get_string(@Settings, "CONFIG_CIFS_USE_SECURE_PORT", "yes")
      )
      SCR.Write(
        path(".sysconfig.novell-cifs.CONFIG_CIFS_USE_OCS_FOR_CREDS"),
        Ops.get_string(@Settings, "CONFIG_CIFS_USE_OCS_FOR_CREDS", "yes")
      )
      SCR.Write(
        path(".sysconfig.novell-cifs.CONFIG_CIFS_LDAP_PORT"),
        Builtins.sformat(
          "%1",
          Ops.get_integer(@Settings, "CONFIG_CIFS_LDAP_PORT", -1)
        )
      )
      SCR.Write(
        path(".sysconfig.novell-cifs.CONFIG_CIFS_PROXY_USER"),
        NovellUtils.Nds2Ldap(
          Ops.get_string(@Settings, "CONFIG_CIFS_PROXY_USER", "")
        )
      )
      SCR.Write(
        path(".sysconfig.novell-cifs.CONFIG_CIFS_EDIR_CONTEXTS"),
        Builtins.mergestring(
          Ops.get_list(@Settings, "CONFIG_CIFS_EDIR_CONTEXTS", []),
          "#"
        )
      )
      #    SCR::Write(.sysconfig.novell-cifs.CONFIG_CIFS_PASSWORD_POLICIES, mergestring(Settings["CONFIG_CIFS_PASSWORD_POLICIES"]:[], "\37"));
      SCR.Write(
        path(".sysconfig.novell-cifs.CONFIG_CIFS_CREATE_NEW_PROXY_USER"),
        Ops.get_string(@Settings, "CONFIG_CIFS_CREATE_NEW_PROXY_USER", "yes")
      )
      SCR.Write(
        path(".sysconfig.novell-cifs.CONFIG_CIFS_USERCONTEXT_RIGHTS"),
        Ops.get_string(@Settings, "CONFIG_CIFS_USERCONTEXT_RIGHTS", "yes")
      )
      SCR.Write(
        path(".sysconfig.novell-cifs.CONFIG_CIFS_SUBTREE_SEARCH"),
        Ops.get_string(@Settings, "CONFIG_CIFS_SUBTREE_SEARCH", "no")
      )

      #    if( !upgradeMode )
      #    {

      cmd = "systemctl restart rsyslog.service"
      ret = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      Builtins.y2milestone(
        "novellcifs.Write:executing:%1, Returned: %2",
        cmd,
        ret
      )

      #Disabling nmb and smb services as novell-samba pattern is removed
      services = ["nmb","smb"]
      Builtins.foreach(services) do |service|
        Service.Stop(service)
        Service.Disable(service)
      end

      #    }
      services = ["novell-vigil","novell-cifs"]

      # Starting novell-vigil service during upgrade
      if Mode.update || Mode.autoupgrade
        Builtins.foreach(services) do |service|
          NovellUtils.SystemdStartorRestartService(service,"yes")
        end
      else
        services = ["novell-cifs"]
        Builtins.foreach(services) do |service|
          NovellUtils.SystemdStartorRestartService(service,"no")    
        end
      end

      SCR.Write(path(".sysconfig.novell-cifs.SERVICE_CONFIGURED"), "yes")

      # Flush the sysconfig file.
      SCR.Write(path(".sysconfig.novell-cifs"), nil)

      Builtins.y2debug("Novell CIFS Services Settings(Write): %1", @Settings)

      return false if Abort()
      Progress.Finish
      Builtins.sleep(sl)

      return false if Abort()
      true
    end

    # Get all novell-cifs settings from the first parameter
    # (For use by autoinstallation.)
    # @param [Hash] settings The YCP structure to be imported.
    # @return [Boolean] True on success
    def Import(settings)
      settings = deep_copy(settings)
      if settings == {}
        @globals_configured = false
        Ops.set(NovellAddonInstall.products_to_configure, "novell-cifs", false)
        return true
      end

      Ops.set(
        @Settings,
        "CONFIG_CIFS_LDAP_SERVER",
        Ops.get_string(settings, "ldap_server", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_CIFS_SERVER_CONTEXT",
        Ops.get_string(settings, "server_context", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_CIFS_USE_SECURE_PORT",
        Ops.get_string(settings, "use_secure_port", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_CIFS_USE_OCS_FOR_CREDS",
        Ops.get_string(settings, "use_ocs_for_credentials", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_CIFS_LDAP_PORT",
        Ops.get_integer(settings, "cifs_ldap_port", -1)
      )
      Ops.set(
        @Settings,
        "CONFIG_CIFS_EDIR_CONTEXTS",
        Ops.get_list(settings, "cifs_edir_contexts", [])
      )
      Ops.set(
        @Settings,
        "CONFIG_CIFS_CREATE_NEW_PROXY_USER",
        Ops.get_string(settings, "create_new_user", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_CIFS_USERCONTEXT_RIGHTS",
        Ops.get_string(settings, "usercontext_rights", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_CIFS_SUBTREE_SEARCH",
        Ops.get_string(settings, "subtree_search", "")
      )
      #    Settings["CONFIG_CIFS_PASSWORD_POLICIES"] = settings["password_policies"]:[];
      @globals_configured = true
      true
    end

    # Dump the novell-cifs settings to a single map
    # (For use by autoinstallation.)
    # @return [Hash] Dumped settings (later acceptable by Import ())
    def Export
      settings = {}

      return settings if NovellUtils.CloneSettingstoAutoinstXml("novell-cifs") == false

      if @auto_clone
        Ops.set(NovellAddonInstall.products_to_configure, "novell-cifs", true)
      end
      if Ops.get(NovellAddonInstall.products_to_configure, "novell-cifs", true)
        Ops.set(
          settings,
          "ldap_server",
          Ops.get_string(@Settings, "CONFIG_CIFS_LDAP_SERVER", "")
        )
        Ops.set(
          settings,
          "server_context",
          Ops.get_string(@Settings, "CONFIG_CIFS_SERVER_CONTEXT", "")
        )
        Ops.set(
          settings,
          "use_secure_port",
          Ops.get_string(@Settings, "CONFIG_CIFS_USE_SECURE_PORT", "")
        )
        Ops.set(
          settings,
          "use_ocs_for_credentials",
          Ops.get_string(@Settings, "CONFIG_CIFS_USE_OCS_FOR_CREDS", "")
        )
        Ops.set(
          settings,
          "cifs_ldap_port",
          Ops.get_integer(@Settings, "CONFIG_CIFS_LDAP_PORT", -1)
        )
        Ops.set(
          settings,
          "cifs_edir_contexts",
          Ops.get_list(@Settings, "CONFIG_CIFS_EDIR_CONTEXTS", [])
        )
        Ops.set(
          settings,
          "create_new_user",
          Ops.get_string(@Settings, "CONFIG_CIFS_CREATE_NEW_PROXY_USER", "yes")
        )
        Ops.set(
          settings,
          "usercontext_rights",
          Ops.get_string(@Settings, "CONFIG_CIFS_USERCONTEXT_RIGHTS", "yes")
        )
        Ops.set(
          settings,
          "subtree_search",
          Ops.get_string(@Settings, "CONFIG_CIFS_SUBTREE_SEARCH", "no")
        ) 
        #        settings["password_policies"] = Settings["CONFIG_CIFS_PASSWORD_POLICIES"]:[];
      end

      deep_copy(settings)
    end

    # Dump all of the novell-cifs settings to a single map
    # (For use by proposal.)
    # @return [Hash] Dumped settings (later acceptable by Import ())
    def ExportSettings
      settings = {}

      return settings if NovellUtils.CloneSettingstoAutoinstXml("novell-cifs") == false

      if @auto_clone
        Ops.set(NovellAddonInstall.products_to_configure, "novell-cifs", true)
      end
      if Ops.get(NovellAddonInstall.products_to_configure, "novell-cifs", true)
        Ops.set(
          settings,
          "ldap_server",
          Ops.get_string(@Settings, "CONFIG_CIFS_LDAP_SERVER", "")
        )
        Ops.set(
          settings,
          "server_context",
          Ops.get_string(@Settings, "CONFIG_CIFS_SERVER_CONTEXT", "")
        )
        Ops.set(
          settings,
          "use_secure_port",
          Ops.get_string(@Settings, "CONFIG_CIFS_USE_SECURE_PORT", "")
        )
        Ops.set(
          settings,
          "use_ocs_for_credentials",
          Ops.get_string(@Settings, "CONFIG_CIFS_USE_OCS_FOR_CREDS", "")
        )
        Ops.set(
          settings,
          "cifs_ldap_port",
          Ops.get_integer(@Settings, "CONFIG_CIFS_LDAP_PORT", -1)
        )
        Ops.set(
          settings,
          "proxy_user",
          Ops.get_string(@Settings, "CONFIG_CIFS_PROXY_USER", "")
        )
        Ops.set(
          settings,
          "proxy_user_password",
          Ops.get_string(@Settings, "CONFIG_CIFS_PROXY_USER_PASSWORD", "")
        )
        Ops.set(
          settings,
          "cifs_edir_contexts",
          Ops.get_list(@Settings, "CONFIG_CIFS_EDIR_CONTEXTS", [])
        )
        Ops.set(
          settings,
          "create_new_user",
          Ops.get_string(@Settings, "CONFIG_CIFS_CREATE_NEW_PROXY_USER", "yes")
        )
        Ops.set(
          settings,
          "usercontext_rights",
          Ops.get_string(@Settings, "CONFIG_CIFS_USERCONTEXT_RIGHTS", "yes")
        )
        Ops.set(
          settings,
          "subtree_search",
          Ops.get_string(@Settings, "CONFIG_CIFS_SUBTREE_SEARCH", "no")
        ) 
        #        settings["password_policies"] = Settings["CONFIG_CIFS_PASSWORD_POLICIES"]:[];
      end

      deep_copy(settings)
    end

    # Tranlate a yes or no string
    # @return translated "yes" or "no" string
    def Trans_yes_no(yesno)
      if Builtins.tolower(yesno) == "yes"
        return _("yes")
      else
        return _("no")
      end
    end


    # Create a textual summary and a list of unconfigured cards
    # @return summary of the current configuration
    def Summary
      summary = ""

      if !@globals_configured
        nc = Summary.NotConfigured
        summary = Summary.AddLine(summary, nc)
      else
        summary = Summary.OpenList(summary)

        summary = Summary.AddListItem(
          summary, # Configuration summary text for autoyast
          Builtins.sformat(
            _("CIFS Server context: %1"),
            Ops.get_string(@Settings, "CONFIG_CIFS_SERVER_CONTEXT", "")
          )
        )

        summary = Summary.AddListItem(
          summary, # Configuration summary text for autoyast
          Builtins.sformat(
            _("LDAP Server: %1"),
            Ops.get_string(@Settings, "CONFIG_CIFS_LDAP_SERVER", "")
          )
        )

        summary = Summary.AddListItem(
          summary, # Configuration summary text for autoyast
          Builtins.sformat(
            _("Use secure channel for configuration: %1"),
            Trans_yes_no(
              Ops.get_string(@Settings, "CONFIG_CIFS_USE_SECURE_PORT", "")
            )
          )
        )

        summary = Summary.AddListItem(
          summary, # Configuration summary text for autoyast
          Builtins.sformat(
            _("Proxy user name: %1"),
            Ops.get_string(@Settings, "CONFIG_CIFS_PROXY_USER", "")
          )
        )

        summary = Summary.AddListItem(
          summary, # Configuration summary text for autoyast
          Builtins.sformat(
            _("CIFS eDirectory contexts: %1"),
            Builtins.mergestring(
              Ops.get_list(@Settings, "CONFIG_CIFS_EDIR_CONTEXTS", []),
              ", "
            )
          )
        )

        summary = Summary.AddListItem(
          summary, # Configuration summary text for autoyast
          Builtins.sformat(
            _("Use OCS for storing the credentials: %1"),
            Trans_yes_no(
              Ops.get_string(@Settings, "CONFIG_CIFS_USE_OCS_FOR_CREDS", "")
            )
          )
        )

        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("LDAP port for CIFS server: %1"),
            Ops.get_integer(@Settings, "CONFIG_CIFS_LDAP_PORT", -1)
          )
        )

        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("Create New Proxy User: %1"),
            Trans_yes_no(
              Ops.get_string(@Settings, "CONFIG_CIFS_CREATE_NEW_PROXY_USER", "")
            )
          )
        )
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("Add Proxy User as Trustee of User Contexts: %1"),
            Trans_yes_no(
              Ops.get_string(@Settings, "CONFIG_CIFS_USERCONTEXT_RIGHTS", "yes")
            )
          )
        )

        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("Enable Subtree Search: %1"),
            Trans_yes_no(
              Ops.get_string(@Settings, "CONFIG_CIFS_SUBTREE_SEARCH", "no")
            )
          )
        )
        #     summary = Summary::AddListItem(
        #	      summary,   /* Configuration summary text for autoyast */

        #	      sformat(_("CIFS Password Policies: %1"),
        #		      mergestring(Settings["CONFIG_CIFS_PASSWORD_POLICIES"]:[], ", ")));

        summary = Summary.CloseList(summary)
      end
      [summary, []]
    end

    # Create an overview table with all configured cards
    # @return table items
    def Overview
      # TODO FIXME: your code here...
      []
    end

    # Return packages needed to be installed and removed during
    # Autoinstallation to insure module has all needed software
    # installed.
    # @return [Hash] with 2 lists.
    def AutoPackages
      # TODO FIXME: your code here...
      { "install" => [], "remove" => [] }
    end

    publish :variable => :auto_clone, :type => "boolean"
    publish :variable => :edir_service_configured, :type => "boolean"
    publish :variable => :cifs_bin_dir, :type => "string"
    publish :variable => :schemaLogPath, :type => "string"
    publish :variable => :xad_packages_installed, :type => "boolean"
    publish :variable => :xadMode, :type => "boolean"
    publish :variable => :xadTestMode, :type => "boolean"
    publish :variable => :upgradeMode, :type => "boolean"
    publish :variable => :reinstallMode, :type => "boolean"
    publish :variable => :Settings, :type => "map"
    publish :variable => :oldSettings, :type => "map"
    publish :function => :Modified, :type => "boolean ()"
    publish :variable => :modified, :type => "boolean"
    publish :variable => :proposal_valid, :type => "boolean"
    publish :variable => :write_only, :type => "boolean"
    publish :variable => :globals_configured, :type => "boolean"
    publish :variable => :AbortFunction, :type => "boolean ()"
    publish :function => :Abort, :type => "boolean ()"
    publish :function => :GetContextData, :type => "list <string> ()"
    publish :function => :RetrievCredentials, :type => "string ()"
    publish :function => :getAdminData, :type => "list <string> ()"
    publish :function => :VerifyUser, :type => "integer (string, string)"
    publish :function => :AttachUserPolicy, :type => "boolean (string)"
    publish :function => :ReadOption, :type => "string (string, string)"
    publish :function => :Read, :type => "boolean ()"
    publish :function => :Write, :type => "boolean ()"
    publish :function => :Import, :type => "boolean (map)"
    publish :function => :Export, :type => "map ()"
    publish :function => :ExportSettings, :type => "map ()"
    publish :function => :Summary, :type => "list ()"
    publish :function => :Overview, :type => "list ()"
    publish :function => :AutoPackages, :type => "map ()"
  end

  NovellCifs = NovellCifsClass.new
  NovellCifs.main
end
